try:
    import pandas as pd
except ImportError:
    raise ImportError('Pandas installation required to import from x§dollop.pandas')

from typing import Union, Generator

PandasType = Union[pd.DataFrame, pd.Series]


def serve(pandas_obj: PandasType, serving_size: int) -> Generator[PandasType, None, None]:
    """
    Read a Pandas object small chunks at a time.

    :param pandas_obj: The Pandas object.
    :param serving_size: The max number of items in each serving.
    :return:
    """
    if isinstance(pandas_obj, (pd.DataFrame, pd.Series)):
        for i in range(0, len(pandas_obj), serving_size):
            yield pandas_obj.iloc[i: i + serving_size]
    else:
        raise NotImplementedError('Dollop pandas.serve only supports objects of DataFrame or Series types.')



import os

from setuptools import setup, find_packages

requirements = []
if os.path.isfile('requirements.txt'):
    with open('requirements.txt', 'r') as file_handler:
        requirements = file_handler.readlines()

setup(
    name='dollop',
    version='1.0.0',
    author='Michael Peter Dunne',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    include_package_data=True,
    install_requires=requirements,
    packages=find_packages(include=['dollop', 'dollop.*']),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
    ],
)



import io
import pathlib

from typing import Any, Generator
from collections.abc import Sequence as SequenceType

from .sequence import serve as serve_sequence

try:
    import pandas as pd
    from .pandas import serve as serve_pandas
    pandas_installed = True
except ImportError:
    pandas_installed = False

from .file import serve as serve_file


def serve(obj: Any, serving_size: int) -> Generator[Any, None, None]:
    """
    Split an iterable of items into a number of smaller iterables, with max serving_size items
    in each. Detect the type automatically and decide how to proceed.

    :param obj: The object that we wish to serve.
    :param serving_size: The max number of items in each outputted subiterable.
    :return:
    """

    if isinstance(obj, SequenceType):
        yield from serve_sequence(obj, serving_size=serving_size)

    elif pandas_installed and isinstance(obj, (pd.DataFrame, pd.Series)):
        yield from serve_pandas(obj, serving_size=serving_size)

    elif isinstance(obj, (io.IOBase, pathlib.Path)):
        yield from serve_file(obj, serving_size=serving_size)

    else:
        raise NotImplementedError(f'Object of type {type(obj)} is not dollopable.')
